from exceptions import NotEnoughResults
from typing import Tuple
from searchresult import BingSearchResult

class BingResponse:
    """Represents a Bing Response (not a search result response)\n
    **DO NOT MANUALLY INITIALIZE THIS CLASS**"""
    def __init__(self, data):
        self.data = data
        self.resulNum = 0

    @property
    def originalQuery(self) -> str:
        """Returns a string if for some reason you forgot what you searched for"""
        return self.data['queryContext']['originalQuery']

    @property
    def matches(self) -> int:
        """Returns the amount of **estimated** matches found for your search query"""
        return self.data['webPages']['totalEstimatedMatches']

    @property
    def searchUrl(self) -> str:
        """Returns the bing search url you would get if you were to type in the query in the bing website directly."""
        return self.data['webPages']['webSearchUrl']

    def getOne(self) -> BingSearchResult:
        """Gets the top search result and returns a BingSearchResult."""
        if not len(self.data['webPages']['value']) >= 1:
            raise NotEnoughResults("There weren't enough search results to give a BingSearchResult.")
        return BingSearchResult((self.data['webPages']['value'][0]))

    def getThree(self) -> Tuple[BingSearchResult, BingSearchResult, BingSearchResult]:
        """Gets the top three search results and returns a tuple of BingSearchResult"""
        if not len(self.data['webPages']['value']) >= 3:
            raise NotEnoughResults("There weren't enough search results to give a BingSearchResult.")
        return (BingSearchResult(self.data['webPages']['value'][0]), BingSearchResult(self.data['webPages']['value'][1]), BingSearchResult(self.data['webPages']['value'][2]))
    
    def getMany(self, amount: int) -> Tuple[BingSearchResult]:
        """Returns x amount of search urls based on the int for ammount passed into the function."""
        if not len(self.data['webPages']['value']) >= amount:
            raise NotEnoughResults("There weren't enough search results to give a BingSearchResult for the specified amount.")
        return ([BingSearchResult(self.data['webPages']['value'][x]) for x in range(1, amount)])



class BingSearchResult:
    """Represents a Bing Search Result (like url, snippet, name, etc)\n
    **DO NOT MANUALLY INITIALIZE THIS CLASS**"""
    def __init__(self, data):
        self.data = data
    
    @property
    def name(self) -> str:
        """Returns the name of the Search Response."""
        return self.data['name']
    
    @property
    def url(self) -> str:
        """Return the url of the Search Response."""
        return self.data['url']
    
    @property
    def isFamilyFriendly(self) -> bool:
        """Returns a bool if the Search Response is family friendly."""
        return self.data['isFamilyFriendly']
    
    @property
    def displayUrl(self) -> str:
        """The display url of the Search Response.
        I don't actually know what this is, I think it is a trimmed version of the url"""
        return self.data['displayUrl']
    
    @property
    def snippet(self) -> str:
        """Returns a snippet of the content on the webpage. It's the description/snippet below the url and name."""
        return self.data['snippet']
    
    @property
    def dateLastCrawled(self) -> str:     # TODO: get this as a datatime.datetime
        """Returns when the page was last crawled by Bing."""
        return self.data['dateLastCrawled']
    
    @property
    def language(self) -> str:
        """Returns what language the page is in."""
        return self.data['language']
    
    @property
    def isNavigational(self) -> bool:
        """I have no idea what this is. Returns a bool if page is navigational."""
        return self.data['isNavigational']



import asyncio
import aiohttp
from bingresponse import BingResponse

class Search:
    """The searching part of asyncbing"""

    def __init__(self, auth: str, session: aiohttp.ClientSession=None):
        self.headers = {'Ocp-Apim-Subscription-Key': auth}
        self.bing = 'https://api.bing.microsoft.com/v7.0/search'
        if not session:
            self.session = asyncio.run(self.session_setup())
        else:
            self.session = session

    async def session_setup(self) -> aiohttp.ClientSession:
        async with aiohttp.ClientSession() as session:
            return session

    async def fetch(self, search: str) -> BingResponse:
        """|coro|
        Searches with the bing api for the search string provided, with the global market set."""
        async with self.session.get(self.bing, headers=self.headers, params={'q': search}) as resp:
            return BingResponse((await resp.json()))

    # "alias"
    async def search(self, *args, **kwargs) -> fetch:
        """|coro|
        An alias for Search.fetch()"""
        return await self.fetch(*args, **kwargs)



from setuptools import setup

setup(
    name='asyncbing',
    version='1.0.2',
    author='TennisBowling',
    author_email='tennisbowling@tennisbowling.com',
    packages=['asyncbing'],
    url='https://github.com/TennisBowling/asyncbing',
    license='LICENSE.md',
    description='A api wrapper for the bing search api.',
    long_description_content_type='text/markdown',
    long_description=open('README.md').read(),
    install_requires=[
        'aiohttp',
        'asyncio',
    ],
)



class BingException(Exception):
    """A generic exception for all bing-related exceptions. You can catch all exceptions with this."""
    pass

class BadAuth(BingException):
    """An invalid authorization was passed in"""
    pass

class NotEnoughResults(BingException):
    """There weren't enough results."""
    pass
